import { GAP_X, GAP_Y, MIDDMLE_X, START_Y } from "../config"

const map = new Map()
map.set('1', 'StartTask')
map.set('2', 'ProcessTask')
map.set('3', 'InterruptTask')
map.set('4', 'EndTask')

function createNode(nodeRaw, level = 0, index = 0, max = 0) {
  let x = 0;
  const y = START_Y + level * GAP_Y
  const _index = index + 1;
  const middle = Math.ceil(max / 2);

  if (_index === middle || max === 0) x = MIDDMLE_X;
  if (_index > middle) x = MIDDMLE_X + (middle * GAP_X)
  if (_index < middle) x = MIDDMLE_X - (middle * GAP_X)

  return ({
    id: nodeRaw.order_process,
    text: nodeRaw.status_name,
    type: 'NormalTask',
    x,
    y,
  })
}

function createStartNode(source) {
  const nodeRaw = source.find(({ action_type }) => action_type === '1');
  const startNode = createNode(nodeRaw);

  return {
    startNode,
    startNodeRaw: nodeRaw
  }
}

function createEdge(sourceNodeId, targetNodeId) {
  return ({
    sourceNodeId,
    targetNodeId,
    type: "dashe-edge"
  })
}

export function generateProcessChart(source, record) {
  const processChart = {
    nodes: [],
    edges: []
  }

  //找到开始节点
  const { startNode, startNodeRaw } = createStartNode(source);
  processChart.nodes.push(startNode);

  //根据开始节点找到其他的节点并转换
  function transform(nodeRaw, level = 0, parent = 0) {
    level++
    const { next_actions } = nodeRaw;
    //从原数据中找到下一个节点的数据
    const nextActionNodeRaw = source.filter(({ status_name }) => next_actions.includes(status_name))
    nextActionNodeRaw.map((raw, index) => {
      const node = createNode(raw, level, index, nextActionNodeRaw.length)
      const edge = createEdge(parent, raw.order_process);
      if(processChart.nodes.filter(({text})=> text === node.text).length === 0) processChart.nodes.push(node)
      processChart.edges.push(edge)
      //存在下个节点 递归生成
      if (raw.next_actions.length > 0) transform(raw, level, raw.order_process)
    });
  }

  transform(startNodeRaw, 0, startNodeRaw.order_process)


  console.log(record)

  //根据审批记录连线
  record.forEach((item, index) => {
    const descIndex = processChart.nodes.findIndex(({ id }) => id === item.order_process);
    if (descIndex >= 0) {
      processChart.nodes[descIndex].type = index === record.length - 1 ? 'ProcessTask' : 'StartTask' 
      processChart.nodes[descIndex].properties = {
        name: item.action_name,
        time: item.addtime
      }
    }

    const sourceEdgeIdx = processChart.edges.findIndex(({ sourceNodeId, targetNodeId }) => {
      return String(sourceNodeId) === String(record[index]?.order_process) && String(targetNodeId) === String(record[index + 1]?.order_process);
    });
    

    if (sourceEdgeIdx >= 0) {
      processChart.edges[sourceEdgeIdx].type = 'polyline'
    }
  })

  return processChart
}
